@@ -1,21 +1,42 @@
-
 package Net::DNS;
-#
-# $Id: DNS.pm 947 2011-10-31 13:17:06Z willem $
-#
-use strict;
-
 
+#
+# $Id: DNS.pm 1064 2012-12-04 08:51:46Z willem $
+#
+use vars qw($SVNVERSION $VERSION);
 BEGIN {
-    eval { require bytes; }
+	$SVNVERSION = (qw$LastChangedRevision: 1064 $)[1];
+	$VERSION = '0.69';
 }
 
 
+=head1 NAME
+
+Net::DNS - Perl interface to the Domain Name System
+
+=head1 SYNOPSIS
+
+    use Net::DNS;
+
+=head1 DESCRIPTION
+
+Net::DNS is a collection of Perl modules that act as a Domain
+Name System (DNS) resolver.  It allows the programmer to perform
+DNS queries that are beyond the capabilities of C<gethostbyname>
+and C<gethostbyaddr>.
+
+The programmer should be somewhat familiar with the format of
+a DNS packet and its various sections.  See RFC 1035 or
+I<DNS and BIND> (Albitz & Liu) for details.
+
+=cut
+
+
+
+
 
 use vars qw(
     $HAVE_XS
-    $VERSION
-    $SVNVERSION
     $DNSSEC
     $DN_EXPAND_ESCAPES
     @ISA
@@ -46,14 +67,6 @@ BEGIN {
     @EXPORT_OK= qw(name2labels wire2presentation rrsort stripdot);
 
 
-
-
-    $VERSION = '0.67';
-    $SVNVERSION = (qw$LastChangedRevision: 947 $)[1];
-
-
-
-
     $HAVE_XS = eval {
 	local $SIG{'__DIE__'} = 'DEFAULT';
 
@@ -79,23 +92,31 @@ BEGIN {
 BEGIN {
 
     $DNSSEC = eval {
-	    local $SIG{'__DIE__'} = 'DEFAULT';
-	    require Net::DNS::SEC;
-	    1
-	    } ? 1 : 0;
-
+		local $SIG{'__DIE__'} = 'DEFAULT';
+		require Net::DNS::SEC;
+		} ? 1 : 0;
 
+	if ( $DNSSEC ) {
+		eval { require Net::DNS::RR::DLV };
+		eval { require Net::DNS::RR::DNSKEY };
+		eval { require Net::DNS::RR::DS };
+		eval { require Net::DNS::RR::KEY };
+		eval { require Net::DNS::RR::NSEC };
+		eval { require Net::DNS::RR::NSEC3 };
+		eval { require Net::DNS::RR::NSEC3PARAM };
+		eval { require Net::DNS::RR::NXT };
+		eval { require Net::DNS::RR::RRSIG };
+		eval { require Net::DNS::RR::SIG };
+	}
 }
 
 
-use Net::DNS::Resolver;
+use strict;
+use Carp;
+use Net::DNS::RR;
 use Net::DNS::Packet;
 use Net::DNS::Update;
-use Net::DNS::Header;
-use Net::DNS::Question;
-use Net::DNS::RR;   # use only after $Net::DNS::DNSSEC has been evaluated
-use Carp;
-
+use Net::DNS::Resolver;
 
 
 #
@@ -160,7 +181,8 @@ use Carp;
     'DHCID'     => 49,      # RFC4701
     'NSEC3'     => 50,      # RFC5155
     'NSEC3PARAM' => 51,     # RFC5155
-# 52-54 are unassigned
+    'TLSA'	=> 52,      # RFC6698
+# 53-54 are unassigned
     'HIP'       => 55,      # RFC5205
     'NINFO'     => 56,      # non-standard					NOT IMPLEMENTED
     'RKEY'      => 57,      # non-standard					NOT IMPLEMENTED
@@ -298,7 +320,7 @@ sub classesbyval {
     'QUERY'        => 0,        # RFC 1035
     'IQUERY'       => 1,        # RFC 1035
     'STATUS'       => 2,        # RFC 1035
-    'NS_NOTIFY_OP' => 4,        # RFC 1996
+    'NOTIFY'       => 4,        # RFC 1996
     'UPDATE'       => 5,        # RFC 2136
 );
 %opcodesbyval = reverse %opcodesbyname;
@@ -362,17 +384,40 @@ sub mx {
 # Auxiliary functions to support dynamic update.
 #
 
-sub yxrrset { return new Net::DNS::RR( shift, 'yxrrset' ); }
+sub yxrrset {
+	my $rr = new Net::DNS::RR(shift);
+	$rr->class('ANY') unless $rr->rdata;
+	return $rr;
+}
 
-sub nxrrset { return new Net::DNS::RR( shift, 'nxrrset' ); }
+sub nxrrset {
+	my $rr = new Net::DNS::RR(shift);
+	$rr->class('NONE');
+	return $rr;
+}
 
-sub yxdomain { return new Net::DNS::RR( shift, 'yxdomain' ); }
+sub yxdomain {
+	my ($domain) = split /\s+/, shift;
+	return new Net::DNS::RR("$domain ANY ANY");
+}
 
-sub nxdomain { return new Net::DNS::RR( shift, 'nxdomain' ); }
+sub nxdomain {
+	my ($domain) = split /\s+/, shift;
+	return new Net::DNS::RR("$domain NONE ANY");
+}
 
-sub rr_add { return new Net::DNS::RR( shift, 'rr_add' ); }
+sub rr_add {
+	my $rr = new Net::DNS::RR(shift);
+	$rr->{ttl} ||= 86400;
+	return $rr;
+}
 
-sub rr_del { return new Net::DNS::RR( shift, 'rr_del' ); }
+sub rr_del {
+	my ( $head, @tail ) = split /\s+/, shift;
+	my $rr = new Net::DNS::RR( scalar @tail > 1 ? "$head @tail": "$head ANY @tail" );
+	$rr->class( $rr->rdata ? 'NONE' : 'ANY' );
+	return $rr;
+}
 
 
 
@@ -403,43 +448,15 @@ sub name2labels {
 
 
 sub wire2presentation {
-    my  $wire=shift;
-    my  $presentation="";
-    my $length=length($wire);
-    # There must be a nice regexp to do this.. but since I failed to
-    # find one I scan the name string until I find a '\', at that time
-    # I start looking forward and do the magic.
+    my $presentation=shift; # Really wire...
 
-    my $i=0;
+    # Prepend these with a backslash
+    $presentation =~ s/(["$();@.\\])/\\$1/g;
 
-    while ($i < $length ){
-	my $char=unpack("x".$i."C1",$wire);
-	if ( $char < 33 || $char > 126 ){
-	    $presentation.= sprintf ("\\%03u" ,$char);
-	}elsif ( $char == ord( "\"" )) {
-	    $presentation.= "\\\"";
-	}elsif ( $char == ord( "\$" )) {
-	    $presentation.= "\\\$";
-	}elsif ( $char == ord( "(" )) {
-	    $presentation.= "\\(";
-	}elsif ( $char == ord( ")" )) {
-	    $presentation.= "\\)";
-	}elsif ( $char == ord( ";" )) {
-	    $presentation.= "\\;";
-	}elsif ( $char == ord( "@" )) {
-	    $presentation.= "\\@";
-	}elsif ( $char == ord( "\\" )) {
-	    $presentation.= "\\\\" ;
-	}elsif ( $char==ord (".") ){
-	    $presentation.= "\\." ;
-	}else{
-	    $presentation.=chr($char) 	;
-	}
-	$i++;
-    }
+    # Convert < 33 and > 126 to \x<\d\d\d>
+    $presentation =~ s/([^\x21-\x7E])/sprintf("\\%03u", ord($1))/eg; 
 
     return $presentation;
-
 }
 
 
@@ -503,11 +520,11 @@ sub presentation2wire {
 #	$successor = $soa->serial(YYYYMMDDxx);
 #
 
-sub SEQUENTIAL() { undef }
+sub SEQUENTIAL { undef }
 
-sub UNIXTIME() { return CORE::time; }
+sub UNIXTIME { return CORE::time; }
 
-sub YYYYMMDDxx() {
+sub YYYYMMDDxx {
 	my ( $dd, $mm, $yy ) = ( localtime )[3 .. 5];
 	return 1900010000 + sprintf '%d%0.2d%0.2d00', $yy, $mm, $dd;
 }
@@ -556,24 +573,7 @@ sub rrsort {
 1;
 __END__
 
-=head1 NAME
 
-Net::DNS - Perl interface to the DNS resolver
-
-=head1 SYNOPSIS
-
-C<use Net::DNS;>
-
-=head1 DESCRIPTION
-
-Net::DNS is a collection of Perl modules that act as a Domain
-Name System (DNS) resolver.  It allows the programmer to perform
-DNS queries that are beyond the capabilities of C<gethostbyname>
-and C<gethostbyaddr>.
-
-The programmer should be somewhat familiar with the format of
-a DNS packet and its various sections.  See RFC 1035 or
-I<DNS and BIND> (Albitz & Liu) for details.
 
 =head2 Resolver Objects
 
@@ -970,22 +970,27 @@ has arrived.
       warn "timed out after $timeout seconds\n";
   }
 
+
 =head1 BUGS
 
 C<Net::DNS> is slow.
 
-For other items to be fixed, please see the TODO file included with
-the source distribution.
+For other items to be fixed, or if you discover a bug in this
+distribution please use the CPAN bug reporting system.
+
 
 =head1 COPYRIGHT
 
-Copyright (c) 1997-2002 Michael Fuhr.
-Portions Copyright (c) 2002-2004 Chris Reinhardt.
-Portions Copyright (c) 2005 Olaf Kolkman (RIPE NCC)
-Portions Copyright (c) 2006 Olaf Kolkman (NLnet Labs)
+Copyright (c)1997-2002 Michael Fuhr.
+Portions Copyright(c)2002-2004 Chris Reinhardt.
+Portions Copyright(c)2005 Olaf Kolkman (RIPE NCC)
+Portions Copyright(c)2006 Olaf Kolkman (NLnet Labs)
+
+All rights reserved.
+
+This program is free software; you may redistribute it and/or
+modify it under the same terms as Perl itself.
 
-All rights reserved.  This program is free software; you may redistribute
-it and/or modify it under the same terms as Perl itself.
 
 =head1 AUTHOR INFORMATION
 
@@ -1006,13 +1011,14 @@ Net::DNS was created by:
 For more information see:
     http://www.net-dns.org/
 
-Stay tuned and syncicate:
+Stay tuned and syndicate:
     http://www.net-dns.org/blog/
 
 =head1 SEE ALSO
 
-L<perl(1)>, L<Net::DNS::Resolver>, L<Net::DNS::Packet>, L<Net::DNS::Update>,
+L<perl>, L<Net::DNS::Resolver>, L<Net::DNS::Packet>, L<Net::DNS::Update>,
 L<Net::DNS::Header>, L<Net::DNS::Question>, L<Net::DNS::RR>, RFC 1035,
 I<DNS and BIND> by Paul Albitz & Cricket Liu
 
 =cut
+
