@@ -1,26 +1,17 @@
 package Net::DNS::Parameters;
 
 #
-# $Id: Parameters.pm 1241 2014-08-11 13:13:59Z willem $
+# $Id: Parameters.pm 1059 2012-11-30 11:19:14Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1241 $)[1];
+$VERSION = (qw$LastChangedRevision: 1059 $)[1];
 
+#
+#	Domain Name System (DNS) Parameters
+#	(last updated 2012-11-27)
+#
 
-################################################
-##
-##	Domain Name System (DNS) Parameters
-##	(last updated 2014-08-06)
-##
-################################################
-
-
-use strict;
-use integer;
-use Carp;
-
-use base qw(Exporter);
-use vars qw(@EXPORT);
+use base Exporter;
 @EXPORT = qw(
 		classbyname classbyval %classbyname
 		typebyname typebyval %typebyname
@@ -29,6 +20,10 @@ use vars qw(@EXPORT);
 		ednsoptionbyname ednsoptionbyval
 		);
 
+use strict;
+use integer;
+use Carp;
+
 
 # Registry: DNS CLASSes
 use vars qw( %classbyname %classbyval );
@@ -40,7 +35,6 @@ use vars qw( %classbyname %classbyval );
 	ANY  => 255,						# RFC1035
 	);
 %classbyval = reverse %classbyname;
-%classbyname = ( '*' => 255, %classbyname, map /\D/ ? lc($_) : $_, %classbyname );
 
 
 # Registry: Resource Record (RR) TYPEs
@@ -85,8 +79,8 @@ use vars qw( %typebyname %typebyval );
 	CERT	   => 37,					# RFC4398
 	A6	   => 38,					# RFC3226 RFC2874 RFC6563
 	DNAME	   => 39,					# RFC6672
-	SINK	   => 40,					# http://tools.ietf.org/html/draft-eastlake-kitchen-sink
-	OPT	   => 41,					# RFC6891 RFC3225
+	SINK	   => 40,					# http://bgp.potaroo.net/ietf/all-ids/draft-eastlake-kitchen-sink-02.txt
+	OPT	   => 41,					# RFC2671 RFC3225
 	APL	   => 42,					# RFC3123
 	DS	   => 43,					# RFC4034 RFC3658
 	SSHFP	   => 44,					# RFC4255
@@ -102,9 +96,8 @@ use vars qw( %typebyname %typebyval );
 	NINFO	   => 56,					#
 	RKEY	   => 57,					#
 	TALINK	   => 58,					#
-	CDS	   => 59,					# RFC-ietf-dnsop-delegation-trust-maintainance-14
-	CDNSKEY	   => 60,					# RFC-ietf-dnsop-delegation-trust-maintainance-14
-	SPF	   => 99,					# RFC7208
+	CDS	   => 59,					#
+	SPF	   => 99,					# RFC4408
 	UINFO	   => 100,					# IANA-Reserved
 	UID	   => 101,					# IANA-Reserved
 	GID	   => 102,					# IANA-Reserved
@@ -113,22 +106,19 @@ use vars qw( %typebyname %typebyval );
 	L32	   => 105,					# RFC6742
 	L64	   => 106,					# RFC6742
 	LP	   => 107,					# RFC6742
-	EUI48	   => 108,					# RFC7043
-	EUI64	   => 109,					# RFC7043
 	TKEY	   => 249,					# RFC2930
 	TSIG	   => 250,					# RFC2845
 	IXFR	   => 251,					# RFC1995
 	AXFR	   => 252,					# RFC1035 RFC5936
 	MAILB	   => 253,					# RFC1035
 	MAILA	   => 254,					# RFC1035
-	ANY	   => 255,					# RFC1035 RFC6895
+	ANY	   => 255,					# RFC1035
 	URI	   => 256,					#
-	CAA	   => 257,					# RFC6844
+	CAA	   => 257,					# RFC-ietf-pkix-caa-15
 	TA	   => 32768,					# http://cameo.library.cmu.edu/ http://www.watson.org/~weiler/INI1999-19.pdf
 	DLV	   => 32769,					# RFC4431
 	);
 %typebyval = reverse %typebyname;
-%typebyname = ( '*' => 255, %typebyname, map /\D/ ? lc($_) : $_, %typebyname );
 
 
 # Registry: DNS OpCodes
@@ -141,7 +131,6 @@ use vars qw( %opcodebyname %opcodebyval );
 	UPDATE => 5,						# RFC2136
 	);
 %opcodebyval = reverse %opcodebyname;
-%opcodebyname = ( NS_NOTIFY_OP => 4, %opcodebyname, map /\D/ ? lc($_) : $_, %opcodebyname );
 
 
 # Registry: DNS RCODEs
@@ -153,13 +142,12 @@ use vars qw( %rcodebyname %rcodebyval );
 	NXDOMAIN => 3,						# RFC1035
 	NOTIMP	 => 4,						# RFC1035
 	REFUSED	 => 5,						# RFC1035
-	YXDOMAIN => 6,						# RFC2136 RFC6672
+	YXDOMAIN => 6,						# RFC2136
 	YXRRSET	 => 7,						# RFC2136
 	NXRRSET	 => 8,						# RFC2136
 	NOTAUTH	 => 9,						# RFC2136
-	NOTAUTH	 => 9,						# RFC2845
 	NOTZONE	 => 10,						# RFC2136
-	BADVERS	 => 16,						# RFC6891
+	BADVERS	 => 16,						# RFC2671
 	BADSIG	 => 16,						# RFC2845
 	BADKEY	 => 17,						# RFC2845
 	BADTIME	 => 18,						# RFC2845
@@ -168,24 +156,17 @@ use vars qw( %rcodebyname %rcodebyval );
 	BADALG	 => 21,						# RFC2930
 	BADTRUNC => 22,						# RFC4635
 	);
-%rcodebyval = reverse( BADSIG => 16, %rcodebyname );
-%rcodebyname = ( %rcodebyname, map /\D/ ? lc($_) : $_, %rcodebyname );
+%rcodebyval = reverse %rcodebyname;
 
 
-# Registry: DNS EDNS0 Option Codes (OPT)
+# Registry: DNS EDNS0 Options
 use vars qw( %ednsoptionbyname %ednsoptionbyval );
 %ednsoptionbyname = (
-	LLQ		=> 1,					# http://files.dns-sd.org/draft-sekar-dns-llq.txt
-	UL		=> 2,					# http://files.dns-sd.org/draft-sekar-dns-ul.txt
-	NSID		=> 3,					# RFC5001
-	DAU		=> 5,					# RFC6975
-	DHU		=> 6,					# RFC6975
-	N3U		=> 7,					# RFC6975
-	'CLIENT-SUBNET' => 8,					# draft-vandergaast-edns-client-subnet
-	EXPIRE		=> 9,					# RFC7314
+	LLQ  => 1,						# http://files.dns-sd.org/draft-sekar-dns-llq.txt
+	UL   => 2,						# http://files.dns-sd.org/draft-sekar-dns-ul.txt
+	NSID => 3,						# RFC5001
 	);
 %ednsoptionbyval = reverse %ednsoptionbyname;
-%ednsoptionbyname = ( %ednsoptionbyname, map /\D/ ? lc($_) : $_, %ednsoptionbyname );
 
 
 # Registry: DNS Header Flags
@@ -212,51 +193,57 @@ use vars qw( %ednsflagbyname );
 # The following functions are wrappers around similarly named hashes.
 
 sub classbyname {
-	my $name = shift;
+	my $name = uc shift;
 
-	$classbyname{$name} || do {
-		confess "unknown type $name" unless $name =~ m/CLASS(\d+)/;
-		my $val = 0 + $1;
-		confess "classbyname( $name ) out of range" if $val > 0xffff;
-		return $val;
-			}
+	return $classbyname{$name} if defined $classbyname{$name};
+
+	confess "unknown class $name" unless $name =~ m/CLASS(\d+)/;
+
+	my $val = 0 + $1;
+	return $val unless $val > 0xffff;
+
+	confess "classbyname( $name ) out of range";
 }
 
 sub classbyval {
 	my $val = shift;
 
-	$classbyval{$val} || do {
-		$val += 0;
-		confess "classbyval( $val ) out of range" if $val > 0xffff;
-		return "CLASS$val";
-			}
+	return $classbyval{$val} if defined $classbyval{$val};
+
+	$val += 0;
+	confess "classbyval( $val ) out of range" if $val > 0xffff;
+
+	return "CLASS$val";
 }
 
 
 sub typebyname {
-	my $name = shift;
+	my $name = uc shift;
 
-	$typebyname{$name} || do {
-		confess "unknown type $name" unless $name =~ m/TYPE(\d+)/;
-		my $val = 0 + $1;
-		confess "typebyname( $name ) out of range" if $val > 0xffff;
-		return $val;
-			}
+	return $typebyname{$name} if defined $typebyname{$name};
+
+	confess "unknown type $name" unless $name =~ m/TYPE(\d+)/;
+
+	my $val = 0 + $1;
+	confess "typebyname( $name ) out of range" if $val > 0xffff;
+
+	return $val ? $val : '00'; ## preserve historical behaviour for TYPE0 [OMK] ##
 }
 
 sub typebyval {
 	my $val = shift;
 
-	$typebyval{$val} || do {
-		$val += 0;
-		confess "typebyval( $val ) out of range" if $val > 0xffff;
-		return "TYPE$val";
-			}
+	return $typebyval{$val} if defined $typebyval{$val};
+
+	$val += 0;
+	confess "typebyval( $val ) out of range" if $val > 0xffff;
+
+	return "TYPE$val";
 }
 
 
 sub opcodebyname {
-	my $name = shift;
+	my $name = uc shift;
 	return $opcodebyname{$name} if defined $opcodebyname{$name};
 	confess "unknown opcode $name";
 }
@@ -268,7 +255,7 @@ sub opcodebyval {
 
 
 sub rcodebyname {
-	my $arg = shift;
+	my $arg = uc shift;
 	return $rcodebyname{$arg} if defined $rcodebyname{$arg};
 	return 0 + $arg if $arg =~ /^\d/;
 	confess "unknown rcode $arg";
@@ -281,7 +268,7 @@ sub rcodebyval {
 
 
 sub ednsoptionbyname {
-	my $arg = shift;
+	my $arg = uc shift;
 	return $ednsoptionbyname{$arg} if defined $ednsoptionbyname{$arg};
 	return 0 + $arg if $arg =~ /^\d/;
 	confess "unknown option $arg";
@@ -312,7 +299,7 @@ __END__
 
 Net::DNS::Parameters is a Perl package representing the DNS parameter
 allocation (key,value) tables as recorded in the definitive registry
-maintained and published by IANA.
+file maintained and published by IANA.
 
 
 =head1 COPYRIGHT
@@ -330,7 +317,7 @@ modify it under the same terms as Perl itself.
 
 
 =head1 SEE ALSO
-
+ 
 L<perl>, L<Net::DNS>,
-L<IANA Registry|http://www.iana.org/assignments/dns-parameters>
+L<IANA dns-parameters|//www.iana.org/assignments/dns-parameters>
 
